<?php
/* 	OpenDb - Open Media Lending Database
	Copyright (C) 2001,2002 by Jason Pell

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

// This must be first - includes config.php
require_once("./include/begin.inc.php");

include_once("./include/language.php");
include_once("./include/theme.php");

include_once("./functions/database.php");
include_once("./functions/auth.php");
include_once("./functions/logging.php");

include_once("./functions/email.php");
include_once("./functions/http.php");
include_once("./functions/utils.php");
include_once("./functions/borrowed_item.php");
include_once("./functions/item.php");
include_once("./functions/datetime.php");
include_once("./functions/item_attribute.php");
include_once("./functions/item_type.php");
include_once("./functions/widgets.php");
include_once("./functions/review.php");
include_once("./functions/listutils.php");
include_once("./functions/status_type.php");
include_once("./functions/HTML_Listing.class.inc");

/**
*/
function handle_reserve($item_id, $instance_no, $borrower_id, &$errors)
{
	global $LANG_VARS;
	global $HTTP_SESSION_VARS;
	global $CONFIG_VARS;
	
	$status_type_r = fetch_status_type_r(fetch_item_s_status_type($item_id, $instance_no));
	if($status_type_r['borrow_ind'] != 'Y')
	{
		$errors = replace_lang_var('s_status_type_desc', $status_type_r['description'], $LANG_VARS['s_status_type_items_cannot_be_borrowed']);
		return FALSE;
	}
	else if(strlen($status_type_r['min_display_user_type'])>0 && !in_array($status_type_r['min_display_user_type'], get_min_user_type_r($HTTP_SESSION_VARS['user_type'])))
	{
		$errors = replace_lang_var('s_status_type_desc', $status_type_r['description'], $LANG_VARS['s_status_type_items_cannot_be_borrowed']);
		return FALSE;
	}
	else if(is_user_owner_of_item($item_id, $instance_no, $borrower_id))
	{
		$errors = $LANG_VARS['cannot_reserve_items_you_own'];
		return FALSE;
	}
	else if(is_item_borrowed_by_user($item_id, $instance_no, $borrower_id))
	{
		$errors = $LANG_VARS['you_have_it_borrowed'];
		return FALSE;
	}
	else if(is_item_reserved_by_user($item_id, $instance_no, $borrower_id))
	{
		$errors = $LANG_VARS['you_have_reservation'];
		return FALSE;
	}
	else if($CONFIG_VARS['borrow.allow_reserve_if_borrowed'] ===FALSE  && is_item_borrowed($item_id, $instance_no))
	{
		$errors = $LANG_VARS['item_is_already_checked_out'];
		return FALSE;
	}
	else if($CONFIG_VARS['borrow.allow_multi_reserve'] === FALSE && is_item_reserved($item_id, $instance_no))
	{
		$errors = $LANG_VARS['item_is_already_reserved'];
		return FALSE;
	}
	else
	{
		// All but the actual reservation of item should occur, when this variable is set to TRUE.
		if($CONFIG_VARS['borrow.reserve_email_only'] !== TRUE)
		{
			$new_borrowed_item_id = reserve_item($item_id, $instance_no, $borrower_id);
			if($new_borrowed_item_id!==FALSE)
			{
				return new_borrowed_item_id;
			}
			else
			{
				return FALSE;
			}
		}
		else
		{
			return TRUE;
		}
	}		
}

/**
* A handle_quick_checkout will only ever handle one request at a time, because
* they are initiated from item_display & listings.php, but not borrow.php
*/
function handle_quick_checkout($item_id, $instance_no, $borrower_id, $borrow_duration, &$errors)
{
	global $LANG_VARS;
	global $HTTP_SESSION_VARS;
	global $CONFIG_VARS;
	
	if(!is_user_valid($borrower_id))
	{
		$errors = replace_lang_var("user_id", $borrower_id, $LANG_VARS['invalid_borrower_user']);
		return FALSE;
	}
	else if(!is_user_allowed_to_borrow($borrower_id))
	{
		$errors = replace_lang_var("user_id", $borrower_id, $LANG_VARS['user_must_be_borrower']);
		return FALSE;
	}
	else if(!is_user_owner_of_item($item_id, $instance_no, $HTTP_SESSION_VARS['user_id']) && 
			!is_user_admin($HTTP_SESSION_VARS['user_id'], $HTTP_SESSION_VARS['user_type']))
	{
		$errors = $LANG_VARS['not_owner_of_item'];
		return FALSE;
	}
	else if(is_item_borrowed($item_id, $instance_no))
	{
		$errors = $LANG_VARS['item_is_already_checked_out'];
		return FALSE;
	}
	
	$status_type_r = fetch_status_type_r(fetch_item_s_status_type($item_id, $instance_no));
	if($status_type_r['borrow_ind'] == 'Y' || $status_type_r['borrow_ind'] == 'N')//Owner operation
	{
		$sequence_number = fetch_borrowed_item_seq_no($item_id, $instance_no, $borrower_id, 'R');
		if($sequence_number !== FALSE)
		{
			if($CONFIG_VARS['borrow.quick_checkout_use_existing_reservation']!==FALSE)
			{
				if(check_out_item($sequence_number, $borrow_duration))
				{
					return $sequence_number;
				}
				else
				{
					return FALSE;
				}
			}
			else
			{
				$errors = replace_lang_var('user_id', $borrower_id, $LANG_VARS['user_has_reservation']);
				return FALSE;
			}
		}//if($sequence_number !== FALSE)
		else
		{	
			$new_borrowed_item_id = quick_check_out_item($item_id, $instance_no, $borrower_id, $borrow_duration);
			if($new_borrowed_item_id!==FALSE)
			{
				return $new_borrowed_item_id;
			}
			else
			{
				return FALSE;
			}
		}
	}		
	else
	{
		$errors = replace_lang_var('s_status_type_desc', $status_type_r['description'], $LANG_VARS['s_status_type_items_cannot_be_borrowed']);
		return FALSE;
	}
}

/**
*/
function handle_cancelreserve($sequence_number, &$errors)
{
	global $LANG_VARS;
	global $HTTP_SESSION_VARS;

	$borrowed_item_r = fetch_borrowed_item_r($sequence_number);
	if ($borrowed_item_r['borrower_id'] !== $HTTP_SESSION_VARS['user_id'] && !is_user_owner_of_item($borrowed_item_r['item_id'], $borrowed_item_r['instance_no'], $HTTP_SESSION_VARS['user_id']))
	{
		$errors = $LANG_VARS['not_allowed_cancel_reserve'];
		return FALSE;
	}
	else if($borrowed_item_r['status'] == 'X')
	{
		$errors = $LANG_VARS['already_cancelled'];
		return FALSE;
	}
	else if($borrowed_item_r['status'] != 'R')
	{
		$errors = $LANG_VARS['borrowed_item_not_found'];
		return FALSE;
	}
	else
	{
		if(cancel_reserve_item($sequence_number))
		{
			return TRUE;
		}
		else
		{
			return FALSE;
		}
	}
}

/**
*/
function handle_checkout($sequence_number, $borrow_duration, &$errors)
{
	global $LANG_VARS;
	global $HTTP_SESSION_VARS;

	$borrowed_item_r = fetch_borrowed_item_r($sequence_number);
	if(!is_user_owner_of_item($borrowed_item_r['item_id'], $borrowed_item_r['instance_no'], $HTTP_SESSION_VARS['user_id']))
	{
		$errors = $LANG_VARS['not_owner_of_item'];
		return FALSE;
	}
	else if(is_item_borrowed($borrowed_item_r['item_id'], $borrowed_item_r['instance_no']))
	{
		$errors = $LANG_VARS['item_is_already_checked_out'];
		return FALSE;
	}
	else if ($borrowed_item_r['status'] != 'R')
	{
		$errors = $LANG_VARS['borrowed_item_not_found'];
		return FALSE;
	}

	// Once a reservation exists, the s_status_type of the item can be changed to a borrow_ind=N type,
	// and the item can still be checked out.  The change of status to a borrow_ind=N, only prevents
	// new reservations from being recorded.
	$status_type_r = fetch_status_type_r(fetch_item_s_status_type($borrowed_item_r['item_id'], $borrowed_item_r['instance_no']));
	if($status_type_r['borrow_ind'] == 'Y' || $status_type_r['borrow_ind'] == 'N')//Existing reservation
	{
		if(check_out_item($sequence_number, $borrow_duration))
		{
			return TRUE;
		}
		else
		{
			return FALSE;
		}
	}
	else
	{
		$errors = replace_lang_var('s_status_type_desc', $status_type_r['description'], $LANG_VARS['s_status_type_items_cannot_be_borrowed']);
		return FALSE;
	}
}

/**
*/
function handle_checkin($sequence_number, &$errors)
{
	global $LANG_VARS;
	global $HTTP_SESSION_VARS;

	$borrowed_item_r = fetch_borrowed_item_r($sequence_number);
	if(!is_user_owner_of_item($borrowed_item_r['item_id'], $borrowed_item_r['instance_no'], $HTTP_SESSION_VARS['user_id']))
	{
		$errors = $LANG_VARS['not_owner_of_item'];
		return FALSE;
	}
	else if ($borrowed_item_r['status'] == 'C')
	{
		$errors = $LANG_VARS['already_checked_in'];
		return FALSE;
	}
	else if ($borrowed_item_r['status'] != 'B')
	{
		$errors = $LANG_VARS['borrowed_item_not_found'];
		return FALSE;
	}
	else
	{
		if(check_in_item($sequence_number))
		{
			return TRUE;
		}
		else
		{
			return FALSE;
		}
	}
}

/**
*/
function handle_reminder($sequence_number, &$errors)
{
	global $LANG_VARS;
	global $HTTP_SESSION_VARS;

	$borrowed_item_r = fetch_borrowed_item_r($sequence_number);
	
	// Administrator should be allowed to send reminders for other peoples items.
	if(!is_user_admin($HTTP_SESSION_VARS['user_id'], $HTTP_SESSION_VARS['user_type']) &&
			!is_user_owner_of_item($borrowed_item_r['item_id'], $borrowed_item_r['instance_no'], $HTTP_SESSION_VARS['user_id']))
	{
		$errors = $LANG_VARS['not_owner_of_item'];
		return FALSE;
	}
	else if ($borrowed_item_r['status'] == 'C')
	{
		$errors = $LANG_VARS['already_checked_in'];
		return FALSE;
	}
	else if ($borrowed_item_r['status'] != 'B')
	{
		$errors = $LANG_VARS['borrowed_item_not_found'];
		return FALSE;
	}
	else
	{
		return TRUE;
	}
}

function handle_extension($sequence_number, $borrow_extension, &$errors)
{
	global $LANG_VARS;
	global $HTTP_SESSION_VARS;

	$borrowed_item_r = fetch_borrowed_item_r($sequence_number);
	if(!is_user_owner_of_item($borrowed_item_r['item_id'], $borrowed_item_r['instance_no'], $HTTP_SESSION_VARS['user_id']))
	{
		$errors = $LANG_VARS['not_owner_of_item'];
		return FALSE;
	}
	else if ($borrowed_item_r['status'] == 'C')
	{
		$errors = $LANG_VARS['already_checked_in'];
		return FALSE;
	}
	else if ($borrowed_item_r['status'] != 'B')
	{
		$errors = $LANG_VARS['borrowed_item_not_found'];
		return FALSE;
	}
	else
	{
		if(item_borrow_duration_extension($sequence_number, $borrow_extension))
			return TRUE;
		else
			return FALSE;
	}
}

/**
* @param $borrowed_item_rs Items that this action will be performed against.  It may actually
* 						be an array of 'sequence_number' values, in which case the borrow
* 						record for the sequence_number will be fetched.
* @param $HTTP_VARS
* */
function more_information_form($op, $borrowed_item_rs, $HTTP_VARS, $email_notification=TRUE)
{
	global $PHP_SELF;
	global $LANG_VARS;
	global $CONFIG_VARS;
	global $HTTP_SESSION_VARS;
	
	$duration_attr_type=NULL;
	$default_borrow_duration=NULL;

	echo("\n<form action=\"$PHP_SELF\" method=\"post\">");
	// In case no detail is required.
	echo("\n<input type=\"hidden\" name=\"more_info_requested\" value=\"true\">");
	
	echo("\n<table cellspacing=1 border=0>");
	
	// Pass all http variables onto next instance...
	// Includes empty fields...
	echo get_url_fields($HTTP_VARS, NULL, NULL);

	// Display the items to be operated on.
	if(is_not_empty_array($borrowed_item_rs))
	{
		// no pagination.
		$HTTP_VARS['items_per_page'] = '';

		$listingObject =& 
					new HTML_Listing(
								$PHP_SELF, 
								$HTTP_VARS); 
		
		$listingObject->setPrintableLinkVisible(FALSE);
		$listingObject->setItemsPerPageControlVisible(FALSE);
		$listingObject->setIncludeHrefLinks(FALSE);
		$listingObject->setIncludeFooter(FALSE);
		$listingObject->setBufferOutput(TRUE);

		$listingObject->startListing();
							
		$listingObject->addHeaderColumn($LANG_VARS['type']);
		$listingObject->addHeaderColumn($LANG_VARS['title']);
		
		if($op == 'reserve' || $op == 'cancel_reserve' || $op == 'quick_check_out')
		{
			$listingObject->addHeaderColumn($LANG_VARS['owner']);
		}
		else if($op == 'check_in')
		{
			$listingObject->addHeaderColumn($LANG_VARS['borrower']);
		}
		
		if($CONFIG_VARS['borrow.duration_support'])
		{
			if($op == 'check_out' || $op == 'quick_check_out')
			{
				$listingObject->addHeaderColumn($LANG_VARS['borrow_duration']);
			}
		}
									
		//initialise
		$max_overdue_duration = NULL;
		$default_borrow_duration = NULL;
		
		reset($borrowed_item_rs);
		while(list(,$borrowed_item_r) = each($borrowed_item_rs))
		{
			$listingObject->startRow();
			
			// If only a sequence_number, we need to fetch the borrow record.
			if(!is_array($borrowed_item_r) && is_numeric($borrowed_item_r))
			{
				$borrowed_item_r = fetch_borrowed_item_r($borrowed_item_r);
			}
			
			$item_r = fetch_item_instance_r($borrowed_item_r['item_id'], $borrowed_item_r['instance_no']);
			
			$listingObject->addItemTypeImageColumn($item_r['s_item_type']);
			$listingObject->addTitleColumn($item_r, FALSE, FALSE);
			
			if($op == 'reserve' || $op == 'cancel_reserve' || $op == 'quick_check_out')
			{
				$listingObject->addUserNameColumn($item_r['owner_id']);
			}
			else if($op == 'check_in')
			{
				$listingObject->addUserNameColumn($borrowed_item_r['borrower_id']);
			}
			
			if($CONFIG_VARS['borrow.duration_support'])
			{
				if($op == 'check_out' || $op == 'quick_check_out')
				{
					if(is_numeric($item_r['borrow_duration']))
					{
						$duration_attr_type_r = fetch_sfieldtype_item_attribute_type_r($item_r['s_item_type'], 'DURATION', TRUE);
						$listingObject->addDisplayColumn(
								$duration_attr_type_r['s_attribute_type'],
								NULL,
								$duration_attr_type_r['display_type'],
								$item_r['borrow_duration']);
					}
					else
					{
						$listingObject->addColumn($LANG_VARS['undefined']);
					}
				}
			}
			
			// While we are here, we are going to calculate the default duration value.
			// We want to choose the least duration value.  If any of the items use a
			// different s_attribute_type for duration, then we should not try to get
			// a default value.  This is indicated by the $duration_attr_type===FALSE
			// if we have encountered a difference.
			if($CONFIG_VARS['borrow.duration_support'] && $duration_attr_type!==FALSE)
			{
				$new_duration_attr_type = fetch_sfieldtype_item_attribute_type($item_r['s_item_type'], 'DURATION');
				if($duration_attr_type == NULL)
					$duration_attr_type = $new_duration_attr_type;
				else if($duration_attr_type !== $new_duration_attr_type)
				{
					// Different s_attribute_type's for DURATION, so cannot display Duration chooser.
					$duration_attr_type=FALSE;
				}

				if($duration_attr_type!==FALSE)
				{
					if($op == 'check_out' || $op == 'quick_check_out')
					{
						// The default borrow duration should be the least amount of days or undefined
						// if no records have a borrow duration.
						if($default_borrow_duration === NULL)
						{
							$default_borrow_duration = $item_r['borrow_duration'];
						}
						else if($default_borrow_duration !== '') // Undefined empty value.
						{
							if(is_numeric($default_borrow_duration) && is_numeric($item_r['borrow_duration']) && 
												$item_r['borrow_duration'] < $default_borrow_duration)
							{
								$default_borrow_duration = $item_r['borrow_duration'];
							}
						}
					}
					else if($op == 'extension')
					{
						if($borrowed_item_r['total_duration'] > $borrowed_item_r['borrow_duration'])
						{
							$tmp_overdue_duration = $borrowed_item_r['total_duration'] - $borrowed_item_r['borrow_duration'];
						}
						
						// We want to get the max overdue duration, so we can give the User granting the extension a
						// default, that will bring all selected items back into non-overdue status.
						if(!is_numeric($max_overdue_duration) || $max_overdue_duration < $tmp_overdue_duration)
						{
							$max_overdue_duration = $tmp_overdue_duration;
						}
					}
				}
			}//if($CONFIG_VARS['borrow.duration_support'] && $duration_attr_type!==FALSE)
			
			$listingObject->endRow();
		}//while(list(,$borrowed_item_r) = each($borrowed_item_rs))
		
		$listingObject->endListing();
		
		echo(
				format_field($LANG_VARS['item(s)'], 
						NULL, 
						$listingObject->getContents()
					)
				);	
		
		// don't need it anymore.
		unset($listingObject);
	}
	
	// Do not display this more information section unless email is enabled.
	if($CONFIG_VARS['email.use_php_mail']!==FALSE && $email_notification!==FALSE)
	{
		echo get_input_field("more_information",
						NULL, 
						$LANG_VARS['more_information'], 
						"textarea(50,10)",
						"N", //compulsory
						NULL,
						TRUE);
	}
	
	// Include a Borrower ID select, minus the current user.
	if($op == 'quick_check_out')
	{
		if(strlen($HTTP_VARS['borrower_id'])==0 || !is_user_allowed_to_borrow($HTTP_VARS['borrower_id']))
		{
			$results = fetch_user_rs(get_borrower_user_types_r(), "fullname", "ASC", FALSE, $HTTP_SESSION_VARS['user_id']);
			if($results)
			{
				echo(
					format_field($LANG_VARS['borrower'], 
							NULL, 
							custom_select('borrower_id', $results, '%fullname% (%user_id%)', 1, NULL, 'user_id')
						)
					);
			}
			else
			{
				echo(
					format_field(
						$LANG_VARS['borrower'],
						NULL,
						$LANG_VARS['no_records_found'])
					);
			}
		}
	}
	
	//Only for check_out/quick_check_out operations - makes no sense otherwise!
	if($CONFIG_VARS['borrow.duration_support']!==FALSE && ($op == 'check_out' || $op == 'quick_check_out' || $op == 'extension'))
	{
		// Display default borrow duration.
		if(strlen($duration_attr_type)>0)
		{
			$duration_attr_type_r = fetch_attribute_type_r($duration_attr_type);
			
			// We have to find the matching DURATION lookup value, which is at least
			// as many days as the max_overdue value, or the highest possible
			// duration value, if none found as large as the $max_overdue_duration
			if($op == 'extension')
			{
				$default_borrow_duration = NULL;
				$result = fetch_attribute_type_lookup_rs($duration_attr_type_r['s_attribute_type'], 'value', 'asc');
				if($result)
				{
					while($lookup_r = mysql_fetch_array($result, MYSQL_ASSOC))
					{	
						if(is_numeric($lookup_r['value']) && (!is_numeric($max_overdue_duration) || (is_numeric($max_overdue_duration) && $max_overdue_duration <= $lookup_r['value'])))
						{
							$default_borrow_duration = $lookup_r['value'];
							break;
						}
						
						// backup, in case we need to use outside while loop
						$lookup_r2 = $lookup_r;					
					}
					mysql_free_result($result);
					
					// If still null, then set to the largest option
					if($default_borrow_duration == NULL)
					{
						$default_borrow_duration = $lookup_r2['value'];
					}
				}
			}
			
			if($op == 'extension')
				$prompt = $duration_attr_type_r['prompt'];
			else if(strlen($LANG_VARS['default_borrow_duration'])>0 && is_array($borrowed_item_rs))
				$prompt = $LANG_VARS['default_borrow_duration'];
			else
				$prompt = $duration_attr_type_r['prompt'];
								
			echo(get_input_field("default_borrow_duration",
							$duration_attr_type_r['s_attribute_type'],
							$prompt, 
							$duration_attr_type_r['input_type'],
							"N",//not compulsory, as we already have default!
							($op!='quick_check_out')?$default_borrow_duration:NULL,
							TRUE));
		
			// Not appropriate for extension operation
			if($op == 'check_out' || $op == 'quick_check_out')
			{
				echo(get_input_field("override_item_duration",
							NULL, 
							$LANG_VARS['override_item_duration'], 
							"simple_checkbox(".($default_borrow_duration===NULL?"CHECKED":"").")",
							"N",
							"Y",
							TRUE));
			}
		}
		else//otherwise tell checkout to use item_instance borrow duration instead.
		{
			if($op == 'check_out' || $op == 'quick_check_out')
			{
				echo("\n<input type=\"hidden\" name=\"override_item_duration\" value=\"N\">");
			}
		}
	}
	
	$button_lang_var = NULL;
	switch($op)
	{
		case 'check_out':
			$button_lang_var = $LANG_VARS['check_out_item(s)'];
			break;
			
		case 'quick_check_out':
			$button_lang_var = $LANG_VARS['check_out_item(s)'];
			break;
		
		case 'check_in':
			$button_lang_var = $LANG_VARS['check_in_item(s)'];
			break;
			
		case 'extension':
			$button_lang_var = $LANG_VARS['borrow_duration_extension(s)'];
			break;
			
		case 'reminder':
			$button_lang_var = $LANG_VARS['send_reminder(s)'];
			break;
			
		case 'reserve':
			$button_lang_var = $LANG_VARS['reserve_item(s)'];
			break;
			
		case 'cancel_reserve':
			$button_lang_var = $LANG_VARS['cancel_reservation(s)'];
			break;
	}
					
	echo("<tr><td colspan=2 align=left><br><input type=submit value=\"".$button_lang_var."\"></td></tr>");
	
	echo("</form>");
	echo("</table>");

	// Display some help information for Duration support functionality.
	if(($op == 'check_out' || $op == 'quick_check_out') && 
				$CONFIG_VARS['borrow.duration_support']!==FALSE && 
				strlen($duration_attr_type)>0 && is_array($borrowed_item_rs))
	{	
		echo(format_help_block($LANG_VARS['check_out_borrow_duration_moreinfo_help']));
	}
}

/**
*/
function display_borrow_results($op, $mode, $heading, $success_intro, $failure_intro, $more_information, $success_item_rs, $failure_item_rs, $email_notification=TRUE)
{
	global $LANG_VARS;
	global $CONFIG_VARS;
	global $HTTP_SESSION_VARS;

	if(is_not_empty_array($success_item_rs))
	{
		if($mode != 'job')
			echo("\n<div class=\"success\">");
			
		echo($success_intro);
		
		if($mode != 'job')
			echo("</div>");
		
		// Sort the items by user, so we can send emails for multiple 
		// items, instead of individually.
		$borrowed_item_user_r = array();
		while(list(,$borrowed_item_r) = each($success_item_rs))
		{
			$item_r = fetch_item_instance_r($borrowed_item_r['item_id'], $borrowed_item_r['instance_no']);

			// Expand title mask.
			$item_r['title'] = expand_item_title_mask($item_r, $CONFIG_VARS['item_display.title_display_mask']);

			// A array of item_entries.
			$item_entry_r['item'] = $item_r;
			$item_entry_r['detail'] = get_borrow_details($op, $item_r, $borrowed_item_r);

			// When reserving or cancelling and the current user is the borrower, we want to
			// send the email to the owner, in all other cases the email should go to the
			// borrower.
			if(($op == 'reserve' || $op == 'cancel_reserve') && 
						$HTTP_SESSION_VARS['user_id'] == $borrowed_item_r['borrower_id'])
			{
				$to_user = $item_r['owner_id'];
			}
			else
			{
				$to_user = $borrowed_item_r['borrower_id'];
			}
			
			// Now add an entry to this user array.
			$borrowed_item_user_r[$to_user][] = $item_entry_r;
		}

		// Now process the user->item structure and send emails.
		$errors = null;
		while(list($to_user,$item_entry_rs) = each($borrowed_item_user_r))
		{
			if($mode != 'job')
				echo("\n<ul>");
			
			while(list(,$item_entry_r) = each($item_entry_rs))
			{
				if($mode != 'job')
					echo("\n<li>");
				else
					echo("\n*    ");
				
				echo(replace_lang_vars(array('display_title'=>$item_entry_r['item']['title'],'item_id'=>$item_entry_r['item']['item_id'],'instance_no'=>$item_entry_r['item']['instance_no']),$LANG_VARS['borrow_item_title_listing']));
				
				if($mode != 'job')
					echo("</li>");
			}
			
			if($mode != 'job')
				echo("\n</ul>");

			if($CONFIG_VARS['email.use_php_mail']!==FALSE && $email_notification!==FALSE)
			{
				// How can the from user be anything but the currently logged in user!
				if(send_notification_email($to_user, $HTTP_SESSION_VARS['user_id'], $heading, $success_intro, $more_information, $item_entry_rs, $errors))
				{
					if($mode != 'job')
						echo("\n<div class=\"smsuccess\">".$LANG_VARS['notication_email_sent']."</div>");
					else
						echo("\n\n".strip_tags($LANG_VARS['notication_email_sent']));
				}
				else
				{
					if($mode != 'job')
						echo("\n<div class=\"smerror\">".$LANG_VARS['notication_email_not_sent']);
					else
						echo("\n\n".strip_tags($LANG_VARS['notication_email_not_sent']));
						
					if(count($errors)>0)
					{
						if($mode != 'job')
							echo("\n<br>");
						else
							echo("\n");
						
						echo($LANG_VARS['errors']);
						
						for($i=0; $i<count($errors); $i++)
						{
							if($mode != 'job')
								echo("\n<br>&nbsp;&nbsp;&nbsp;&nbsp;<i>");
							else
								echo("\n*    ");
							
							echo($errors[$i]);
							
							if($mode != 'job')
								echo("</i>");
						}
					}
					
					if($mode != 'job')
						echo("\n</div>");
				}
			}
		}
		
		if($mode != 'job')
			echo("\n<br><br>");
		else
			echo("\n\n");
	}

	// No emails here.
	if(is_not_empty_array($failure_item_rs))
	{
		if($mode != 'job')
			echo("\n<div class=\"success\">");
		
		echo($failure_intro);
		
		if($mode != 'job')
		{
			echo("</div>");
			echo("\n<ul>");
		}
		
		while(list(,$borrowed_item_r) = each($failure_item_rs))
		{
			$item_r = fetch_item_instance_r($borrowed_item_r['item_id'], $borrowed_item_r['instance_no']);
			
			// Expand title mask.
			$item_r['title'] = expand_item_title_mask($item_r, $CONFIG_VARS['item_display.title_display_mask']);
			
			if($mode != 'job')
				echo("\n<li>");
			
			echo(replace_lang_vars(array('display_title'=>$item_r['title'],'item_id'=>$item_r['item_id'],'instance_no'=>$item_r['instance_no']),$LANG_VARS['borrow_item_title_listing']));
			
			// Now display any errors if present.
			if(strlen($borrowed_item_r['errors'])>0)
			{
				if($mode != 'job')
					echo("\n<br><div class=\"smerror\">");
				else
					echo("\n");
					
				echo(replace_lang_var("error", $borrowed_item_r['errors'], $LANG_VARS['borrow_error_detail']));
				
				if($mode != 'job')
					echo("</div>");
			}
			
			if($mode != 'job')
				echo("\n</li>");
		}
		
		if($mode != 'job')
			echo("\n</ul>");
	}
}

/**
	This will format a status line for the borrowed_item_r passed as parameter.

	$op == check_in
		$LANG_VARS['check_in_detail'] = 'Total/Overdue: {total_duration} day(s)/{overdue_duration} day(s)';
	$op == check_out || quick_check_out
		$LANG_VARS['due_date_detail'] = 'Due: {date} ({borrow_duration} day(s))';
	$op == reminder
		$LANG_VARS['due_date_detail'] = 'Due: {date} ({borrow_duration} day(s))';
		$LANG_VARS['check_in_detail'] = 'Total/Overdue: {total_duration} day(s)/{overdue_duration} day(s)';
*/
function get_borrow_details($op, $item_r, $borrowed_item_r)
{
	global $CONFIG_VARS;
	global $LANG_VARS;

	if($op == 'check_out' || $op == 'quick_check_out' || $op == 'reminder' || $op == 'admin_send_reminders' || $op == 'extension')
	{
		if($borrowed_item_r['due_date']>0 && is_numeric($borrowed_item_r['borrow_duration']) && $borrowed_item_r['borrow_duration']>0)
		{
			$details_r[] = replace_lang_var('borrow_duration', $borrowed_item_r['borrow_duration'],$LANG_VARS['due_duration_detail']);
			$details_r[] = replace_lang_var('date', get_localised_timestamp($CONFIG_VARS['borrow.date_mask'], $borrowed_item_r['due_date']), $LANG_VARS['due_date_detail']);
		}
	}

	if($op == 'check_in' || $op == 'reminder' || $op == 'admin_send_reminders')
	{
		if($borrowed_item_r['total_duration']>0 && is_numeric($borrowed_item_r['borrow_duration']) && $borrowed_item_r['borrow_duration']>0)
		{
			$details_r[] = replace_lang_var('total_duration', $borrowed_item_r['total_duration'], $LANG_VARS['total_duration_detail']);
			
			if($borrowed_item_r['total_duration']>$borrowed_item_r['borrow_duration'])
			{
				$overdue_duration = $borrowed_item_r['total_duration'] - $borrowed_item_r['borrow_duration'];
				$details_r[] = replace_lang_var('overdue_duration', $overdue_duration, $LANG_VARS['overdue_duration_detail']);
			}
		}
	}

	return $details_r;
}

/**
	The $borrow_item_r array format:
		$item_r
		$details
*/
function send_notification_email($to_user, $from_user, $heading, $introduction, $more_information, $item_entry_rs, &$errors)
{
	global $LANG_VARS;
	global $CONFIG_VARS;

	// Format the entire message.
	$message =  replace_lang_var("fullname", fetch_user_name($to_user), expand_langvar_newlines($LANG_VARS['to_user_email_intro'])).
				"\n\n".
				$introduction;
	
	while(list(,$item_entry_r) = each($item_entry_rs))
	{
		$message .= "\n*    ".replace_lang_vars(array('display_title'=>$item_entry_r['item']['title'],'item_id'=>$item_entry_r['item']['item_id'],'instance_no'=>$item_entry_r['item']['instance_no']), $LANG_VARS['borrow_item_title_listing']);

		// Add any item Borrow (overdue,due,reminder,etc) details here.
		while(list(,$detail) = @each($item_entry_r['detail']))
		{
			$message .= "\n     - ".$detail;
		}
	}

	if(strlen($more_information)>0)
	{
		$message .= "\n\n\n\n\n".
				$LANG_VARS['more_information'].
				"\n";

		// Add title underline...
		for($i=0;$i<strlen($LANG_VARS['more_information']);$i++)
			$message .= "-";

		$message .= "\n".
					$more_information.
					"\n\n";
	}

	// Send the mail!
	$results = opendb_user_email(
					$to_user, 
					$from_user, 
					$heading, 
					$message);
	if($results === TRUE)
		return TRUE;
	else
	{
		$errors = $results;
		return FALSE;
	}			
}

function add_errors_to_borrowed_item_r($borrowed_item_r, $errors)
{
	$borrowed_item_r['errors'] = $errors;
	return $borrowed_item_r;
}

if($CONFIG_VARS['site.enable']!==FALSE)
{
	session_start();
	if (is_opendb_valid_session())
	{
		if($CONFIG_VARS['borrow.enable']!==FALSE)
		{
			if($HTTP_VARS['op'] == 'admin_send_reminders' && $HTTP_VARS['mode'] == 'job')
			{
				@set_time_limit(600);

				header("Content-Type: text/plain");
				echo $LANG_VARS['check_in_reminder'];
				
				for($i=0;$i<strlen($LANG_VARS['check_in_reminder']);$i++)
					$dashed_line .= "-";
				
				echo("\n$dashed_line\n\n");
				
				if(is_user_admin($HTTP_SESSION_VARS['user_id'], $HTTP_SESSION_VARS['user_type']))
				{
					if($CONFIG_VARS['email.use_php_mail'] !== FALSE)
					{
						$results = fetch_reminder_borrowed_item_rs($CONFIG_VARS['borrow.reminder.duration_range']);
						if($results)
						{
							while($borrowed_item_r = mysql_fetch_array($results, MYSQL_ASSOC))
							{
								if(handle_reminder($borrowed_item_r['sequence_number'], $errors))
									$success_items_rs[] = $borrowed_item_r;
								else
								{
									$borrowed_item_r['errors'] = $errors;
									$failure_items_rs[] = $borrowed_item_r;
								}
							}
							mysql_free_result($results);
							
							display_borrow_results(
										$HTTP_VARS['op'],
										$HTTP_VARS['mode'],
										$LANG_VARS['check_in_reminder'],
										$LANG_VARS['check_in_reminder_for_items'],
										$LANG_VARS['check_in_reminder_not_for_items'],
	 								    NULL, //$HTTP_VARS['more_information'],
										$success_items_rs, 
										$failure_items_rs,
										TRUE);
						}
						else
						{
							echo $LANG_VARS['no_records_found'];
						}
					}
					else
					{
						echo $LANG_VARS['operation_not_available'];
					}
				}
				else
				{
					echo $LANG_VARS['not_authorized_to_page'];
				}
			}
			else if(is_user_allowed_to_borrow($HTTP_SESSION_VARS['user_id'], $HTTP_SESSION_VARS['user_type'])) // don't allow guests in!
			{
				// So we can get errors back!
				$errors = null;
				
				if( $HTTP_VARS['op'] == 'reserve_all' || $HTTP_VARS['op'] == 'reserve')
				{	
					echo _theme_header($LANG_VARS['item_reservation']);
					echo("<h2>".$LANG_VARS['item_reservation']."</h2>");
				
					// If $op == 'reserve_all', then the reserve_basket_listing session variable should be taken into account, 
					// otherwise it should be ignored.
					if($HTTP_VARS['op'] == 'reserve_all')
					{
						$HTTP_VARS['item_id_instance_no'] = $HTTP_SESSION_VARS['reserve_basket_listing'];
						
						// Reset - so we will not process the session list again.
						$HTTP_VARS['op'] = 'reserve';
					}
					else if(is_checked_listing_vars($HTTP_VARS, 'item_id_instance_no'))
					{
						$listingObject = new HTML_Listing($PHP_SELF, $HTTP_VARS);
						$listingObject->setCheckBoxColumnAndInitList('item_id_instance_no');
						
						$HTTP_VARS['item_id_instance_no'] = $listingObject->getCheckedList();
					
						// Don't need the Listings object anymore.
						unset($listingObject);
					}
					
					if(is_not_empty_array($HTTP_VARS['item_id_instance_no']))
					{
						// Format of $item_id_instance_no {item_id}_{instance_no}
						while ( list(,$item_id_instance_no) = each($HTTP_VARS['item_id_instance_no']))
						{
							$item_id_instance_no_r = get_item_id_and_instance_no($item_id_instance_no);
							if(is_not_empty_array($item_id_instance_no_r))
								$reserve_item_rs[] = $item_id_instance_no_r;
						}
					}
					else if(is_numeric($HTTP_VARS['item_id']) && is_numeric($HTTP_VARS['instance_no']))
					{
						if(is_exists_item_instance($HTTP_VARS['item_id'], $HTTP_VARS['instance_no']))
						{
							$reserve_item_rs[] = array('item_id'=>$HTTP_VARS['item_id'],'instance_no'=>$HTTP_VARS['instance_no']);
						}
					}
					
					if(is_array($reserve_item_rs))
					{
						// There is no point in providing a More Information form, unless we either have use of php email,
						// or we are in checkout mode.
						if($CONFIG_VARS['borrow.reserve_more_information'] && $HTTP_VARS['more_info_requested'] != 'true')//even if $message_done is empty it is still set!
						{
							more_information_form('reserve', 
											$reserve_item_rs, 
											$HTTP_VARS,
											$CONFIG_VARS['borrow.reserve_email_notification']);
						}
						else
						{
							// Now get rid of the items that have been processed.
							if(is_not_empty_array($HTTP_SESSION_VARS['reserve_basket_listing']))
							{
								// Remove the items which have been processed.
								$HTTP_SESSION_VARS['reserve_basket_listing'] = 
															minus_array_values(
																$HTTP_SESSION_VARS['reserve_basket_listing'], 
																$HTTP_VARS['item_id_instance_no']);
	
								// If still items left, better update the array.						
								if(is_not_empty_array($HTTP_SESSION_VARS['reserve_basket_listing']))
								{
									if(is_register_globals_enabled())
									{
										session_register('reserve_basket_listing');
										$reserve_basket_listing = $HTTP_SESSION_VARS['reserve_basket_listing']; 
									}
								}
								else
								{	
									if(is_register_globals_enabled())
									{
										session_unregister('reserve_basket_listing');
									}
								}
							}
						
							while ( list(,$reserve_item_r) = @each($reserve_item_rs))
							{
								// In case someone is trying to pass invalid item_id/instance_no combo's
								if(is_exists_item_instance($reserve_item_r['item_id'],$reserve_item_r['instance_no']))
								{
									if(($new_borrowed_item_id = handle_reserve($reserve_item_r['item_id'], $reserve_item_r['instance_no'], $HTTP_SESSION_VARS['user_id'], $errors)) !== FALSE)// This allows reserve to support calls from borrow.php, item_display.php or listings.php
										$success_items_rs[] = array(item_id=>$reserve_item_r['item_id'],instance_no=>$reserve_item_r['instance_no'],borrower_id=>$HTTP_SESSION_VARS['user_id']);	
									else
										$failure_items_rs[] = array(item_id=>$reserve_item_r['item_id'],instance_no=>$reserve_item_r['instance_no'],borrower_id=>$HTTP_SESSION_VARS['user_id'],errors=>$errors);
								}
							}
						
							display_borrow_results(
										$HTTP_VARS['op'],
										$HTTP_VARS['mode'],
										$LANG_VARS['item_reservation'],
										$LANG_VARS['items_have_been_registered'], 
										$LANG_VARS['items_have_not_been_registered'],
	 								    $HTTP_VARS['more_information'],
										$success_items_rs, 
										$failure_items_rs,
										$CONFIG_VARS['borrow.reserve_email_notification']);
						}
					}
					else
					{
						echo _theme_error($LANG_VARS['undefined_error']);
					}
				}
				else if($HTTP_VARS['op'] == 'cancel_reserve')
				{
					echo _theme_header($LANG_VARS['item_cancel_reservation']);
					echo("<h2>".$LANG_VARS['item_cancel_reservation']."</h2>");
						
					// So we can process only sequence_numbers
					if(is_checked_listing_vars($HTTP_VARS, 'sequence_number'))
					{
						$listingObject = new HTML_Listing($PHP_SELF, $HTTP_VARS);
						$listingObject->setCheckBoxColumnAndInitList('sequence_number');
						
						$HTTP_VARS['sequence_number'] = $listingObject->getCheckedList();
						$sequence_number_r = $HTTP_VARS['sequence_number'];
										
						// Don't need the HTML_Listings object anymore.
						unset($listingObject);
					}
					else if(is_numeric($HTTP_VARS['sequence_number']))
					{
						$sequence_number_r[] = $HTTP_VARS['sequence_number']; //convert to array here.
					}
					else if(is_numeric($HTTP_VARS['item_id']) && is_numeric($HTTP_VARS['instance_no']))//In this case the $borrower_id has to be the current user, no one else can cancel using this function!
					{
						$sequence_number_r[] = fetch_borrowed_item_seq_no($HTTP_VARS['item_id'], $HTTP_VARS['instance_no'], $HTTP_SESSION_VARS['user_id'], 'R');
					}
					
					if(is_array($sequence_number_r))
					{
						// There is no point in providing a More Information form, unless we either have use of php email,
						// or we are in checkout mode.
						if($CONFIG_VARS['borrow.cancel_more_information'] && $HTTP_VARS['more_info_requested'] != 'true')
						{
							more_information_form('cancel_reserve', 
											$sequence_number_r, 
											$HTTP_VARS,
											$CONFIG_VARS['borrow.cancel_email_notification']);
						}
						else
						{
							while ( list(,$sequence_number) = @each($sequence_number_r))
							{
								// This allows cancel-reserve to support calls from borrow.php, item_display.php or listings.php
								if(handle_cancelreserve($sequence_number, $errors))
									$success_items_rs[] = fetch_borrowed_item_r($sequence_number);
								else
									$failure_items_rs[] = add_errors_to_borrowed_item_r(fetch_borrowed_item_pk_r($sequence_number), $errors);
							}
				
							display_borrow_results(
										$HTTP_VARS['op'],
										$HTTP_VARS['mode'],
										$LANG_VARS['item_cancel_reservation'],
										$LANG_VARS['reserve_items_have_been_cancelled'], 
										$LANG_VARS['reserve_items_have_not_been_cancelled'],
	 									$HTTP_VARS['more_information'],
										$success_items_rs, 
										$failure_items_rs,
										$CONFIG_VARS['borrow.cancel_email_notification']);
						}
					}
					else
					{
						echo _theme_error($LANG_VARS['undefined_error']);
					}
				}
				else if($HTTP_VARS['op'] == 'check_out')
				{
					echo _theme_header($LANG_VARS['item_check_out']);
					echo("<h2>".$LANG_VARS['item_check_out']."</h2>");
					
					// It is easier to assume an array in all cases.	
					if(is_checked_listing_vars($HTTP_VARS, 'sequence_number'))
					{
						$listingObject = new HTML_Listing($PHP_SELF, $HTTP_VARS);
						$listingObject->setCheckBoxColumnAndInitList('sequence_number');
						
						$HTTP_VARS['sequence_number'] = $listingObject->getCheckedList();
						$sequence_number_r = $HTTP_VARS['sequence_number'];
										
						// Don't need the HTML_Listings object anymore.
						unset($listingObject);
					}
					else if(is_numeric($HTTP_VARS['sequence_number']))
					{
						$sequence_number_r[] = $HTTP_VARS['sequence_number']; //convert to array here.
					}
					
					if(is_array($sequence_number_r))
					{
						// There is no point in providing a More Information form, unless we either have use of php email,
						// or we are in checkout mode.
						if($CONFIG_VARS['borrow.checkout_more_information'] && $HTTP_VARS['more_info_requested'] != 'true')
						{	
							more_information_form('check_out', 
										$sequence_number_r, 
										$HTTP_VARS,
										$CONFIG_VARS['borrow.checkout_email_notification']);
						}
						else
						{
							while ( list(,$sequence_number) = each($sequence_number_r))
							{
								$borrow_duration = NULL;
							
								// The More Information form was not presented
								// So we need to get the default duration from the item table.
								if($HTTP_VARS['more_info_requested'] != 'true')
								{
									$borrowed_item_r = fetch_borrowed_item_pk_r($sequence_number);
									if(is_not_empty_array($borrowed_item_r))
									{
										$item_r = fetch_item_instance_r($borrowed_item_r['item_id'], $borrowed_item_r['instance_no']);
										if(is_not_empty_array($item_r))
										{
											$borrow_duration = $item_r['borrow_duration'];
										}
									}
								}
								else // else more information form presented, so we have to factor in overriding borrow duration.
								{
									// In this case a duration of '' is supported
									if($HTTP_VARS['override_item_duration'] != 'Y')
									{
										$borrowed_item_r = fetch_borrowed_item_pk_r($sequence_number);
										if(is_not_empty_array($borrowed_item_r))
										{
											$item_r = fetch_item_instance_r($borrowed_item_r['item_id'], $borrowed_item_r['instance_no']);
											if(is_not_empty_array($item_r))
											{
												$borrow_duration = $item_r['borrow_duration'];
											}
										}
										$borrow_duration = $item_r['borrow_duration'];
									}
								
									if(!is_numeric($borrow_duration))
									{
										$borrow_duration = $HTTP_VARS['default_borrow_duration'];
									}
								}
	
								if(handle_checkout($sequence_number, $borrow_duration, $errors))
									$success_items_rs[] = fetch_borrowed_item_r($sequence_number,TRUE);
								else
									$failure_items_rs[] = add_errors_to_borrowed_item_r(fetch_borrowed_item_pk_r($sequence_number), $errors);
							}
	
							display_borrow_results(
											$HTTP_VARS['op'],
											$HTTP_VARS['mode'],
											$LANG_VARS['item_check_out'],
											$LANG_VARS['items_have_been_checked_out'], 
											$LANG_VARS['items_have_not_been_checked_out'],
	 									    $HTTP_VARS['more_information'],
											$success_items_rs, 
											$failure_items_rs,
											$CONFIG_VARS['borrow.checkout_email_notification']);
						}
					}
					else
					{
						echo _theme_error($LANG_VARS['undefined_error']);
					}
				}
				else if($HTTP_VARS['op'] == 'quick_check_out')
				{
					if(strlen($HTTP_VARS['borrower_id'])==0 || !is_user_allowed_to_borrow($HTTP_VARS['borrower_id']))
					{
						echo _theme_header($LANG_VARS['item_quick_check_out']);
						echo("<h2>".$LANG_VARS['item_quick_check_out']."</h2>");
					}
					else
					{
						$page_title = replace_lang_vars(array('user_id'=>$HTTP_VARS['borrower_id'], 'fullname'=>fetch_user_name($HTTP_VARS['borrower_id'])), $LANG_VARS['item_quick_check_out_for_fullname']);
						echo _theme_header($page_title);
						echo("<h2>".$page_title."</h2>");
					}
					
					if(is_not_empty_array($HTTP_VARS['checkout_item_instance_rs']))
					{
						while(list(,$item_id_and_instance_no) = each($HTTP_VARS['checkout_item_instance_rs']))
						{
							if(strlen($item_id_and_instance_no)>0)
							{
								$item_id_and_instance_no_r = get_item_id_and_instance_no($item_id_and_instance_no);
								if(is_not_empty_array($item_id_and_instance_no_r))
								{
									$checkout_item_r = fetch_item_instance_r($item_id_and_instance_no_r['item_id'], $item_id_and_instance_no_r['instance_no']);
									if(is_array($checkout_item_r))
									{
										$checkout_item_rs[] = $checkout_item_r;
									}
								}
							}
						}
					}
					else if(is_numeric($HTTP_VARS['item_id']) && is_numeric($HTTP_VARS['instance_no']))
					{
						if(is_exists_item_instance($HTTP_VARS['item_id'], $HTTP_VARS['instance_no']))
						{
							$checkout_item_rs[] = fetch_item_instance_r($HTTP_VARS['item_id'], $HTTP_VARS['instance_no']);;
						}
					}
					
					if(is_array($checkout_item_rs))
					{
						// In the case of quick check, we always want a more information form, because this is
						// where the 'borrower_id' is selected.
						if($HTTP_VARS['more_info_requested'] != 'true')
						{	
							more_information_form(
										'quick_check_out',
										$checkout_item_rs,
										$HTTP_VARS,
										$CONFIG_VARS['borrow.quick_checkout_email_notification']);
						}
						else
						{
							while ( list(,$checkout_item_r) = @each($checkout_item_rs))
							{
								// In case someone is trying to pass invalid item_id/instance_no combo's
								if(is_exists_item_instance($checkout_item_r['item_id'],$checkout_item_r['instance_no']))
								{
									$borrow_duration = NULL;
									if($HTTP_VARS['override_item_duration'] != 'Y' && is_numeric($checkout_item_r['borrow_duration']))
										$borrow_duration = $checkout_item_r['borrow_duration'];
									else
										$borrow_duration = $HTTP_VARS['default_borrow_duration'];	
									
									// This allows reserve to support calls from borrow.php, item_display.php or listings.php
									$new_borrowed_item_id = handle_quick_checkout($checkout_item_r['item_id'], $checkout_item_r['instance_no'], $HTTP_VARS['borrower_id'], $borrow_duration, $errors);
									if($new_borrowed_item_id!==FALSE)
										$success_items_rs[] = fetch_borrowed_item_r($new_borrowed_item_id,TRUE);
									else
										$failure_items_rs[] = array(item_id=>$checkout_item_r['item_id'],instance_no=>$checkout_item_r['instance_no'],borrower_id=>$HTTP_VARS['borrower_id'],errors=>$errors);
								}							
							}
	
							display_borrow_results(
										$HTTP_VARS['op'],
										$HTTP_VARS['mode'],
										$LANG_VARS['item_quick_check_out'],
										$LANG_VARS['items_have_been_checked_out'], 
										$LANG_VARS['items_have_not_been_checked_out'],
	 								    $HTTP_VARS['more_information'],
										$success_items_rs, 
										$failure_items_rs,
										$CONFIG_VARS['borrow.quick_checkout_email_notification']);
						}
					}
					else
					{
						echo _theme_error($LANG_VARS['undefined_error']);
					}
				}
				else if($HTTP_VARS['op'] == 'check_in')
				{
					echo _theme_header($LANG_VARS['item_check_in']);		
					echo("<h2>".$LANG_VARS['item_check_in']."</h2>");
					
					// It is easier to assume an array in all cases.	
					if(is_checked_listing_vars($HTTP_VARS, 'sequence_number'))
					{
						$listingObject = new HTML_Listing($PHP_SELF, $HTTP_VARS);
						$listingObject->setCheckBoxColumnAndInitList('sequence_number');
						
						$HTTP_VARS['sequence_number'] = $listingObject->getCheckedList();
						$sequence_number_r = $HTTP_VARS['sequence_number'];
										
						// Don't need the HTML_Listings object anymore.
						unset($listingObject);
					}
					else if(is_numeric($HTTP_VARS['sequence_number']))//is_numeric
					{
						$sequence_number_r[] = $HTTP_VARS['sequence_number']; //convert to array here.
					}
					
					if(is_array($sequence_number_r))
					{	
						// There is no point in providing a More Information form, unless we either have use of php email,
						// or we are in checkout mode.
						if($CONFIG_VARS['borrow.checkin_more_information'] && $HTTP_VARS['more_info_requested'] != 'true')
						{
							more_information_form('check_in', 
										$sequence_number_r, 
										$HTTP_VARS,
										$CONFIG_VARS['borrow.checkin_email_notification']);
						}
						else
						{
							while ( list(,$sequence_number) = @each($sequence_number_r))
							{
								if(handle_checkin($sequence_number, $HTTP_SESSION_VARS['user_id'], $errors))
									$success_items_rs[] = fetch_borrowed_item_r($sequence_number);
								else
									$failure_items_rs[] = add_errors_to_borrowed_item_r(fetch_borrowed_item_pk_r($sequence_number), $errors);
							}
	
							display_borrow_results(
										$HTTP_VARS['op'],
										$HTTP_VARS['mode'],
										$LANG_VARS['item_check_in'],
										$LANG_VARS['items_have_been_checked_in'], 
										$LANG_VARS['items_have_not_been_checked_in'],
	 								    $HTTP_VARS['more_information'],
										$success_items_rs, 
										$failure_items_rs,
										$CONFIG_VARS['borrow.checkin_email_notification']);
						}
					}
					else
					{
						echo _theme_error($LANG_VARS['undefined_error']);
					}
				}
				else if($HTTP_VARS['op'] == 'reminder')
				{
					echo _theme_header($LANG_VARS['check_in_reminder']);
					echo("<h2>".$LANG_VARS['check_in_reminder']."</h2>");
					
					if($CONFIG_VARS['email.use_php_mail'] !== FALSE)
					{
						// It is easier to assume an array in all cases.	
						if(is_checked_listing_vars($HTTP_VARS, 'sequence_number'))
						{
							$listingObject = new HTML_Listing($PHP_SELF, $HTTP_VARS);
							$listingObject->setCheckBoxColumnAndInitList('sequence_number');
						
							$HTTP_VARS['sequence_number'] = $listingObject->getCheckedList();
							$sequence_number_r = $HTTP_VARS['sequence_number'];
										
							// Don't need the HTML_Listings object anymore.
							unset($listingObject);
						}
						else if(is_numeric($HTTP_VARS['sequence_number']))//is_numeric
						{
							$sequence_number_r[] = $HTTP_VARS['sequence_number']; //convert to array here.
						}
					
						if(is_array($sequence_number_r))
						{	
							// If we are providing for custom email to go along with the message then we need to take care of it here.
							if($CONFIG_VARS['borrow.reminder_more_information'] && $HTTP_VARS['more_info_requested'] != 'true')
							{
								more_information_form('reminder', 
											$sequence_number_r, 
											$HTTP_VARS,
											TRUE);
							}
							else
							{
								while ( list(,$sequence_number) = @each($sequence_number_r))
								{
									if(handle_reminder($sequence_number, $errors))
										$success_items_rs[] = fetch_borrowed_item_r($sequence_number,TRUE);
									else
										$failure_items_rs[] = add_errors_to_borrowed_item_r(fetch_borrowed_item_pk_r($sequence_number), $errors);
								}
	
								display_borrow_results(
										$HTTP_VARS['op'],
										$HTTP_VARS['mode'],
										$LANG_VARS['check_in_reminder'],
										$LANG_VARS['check_in_reminder_for_items'],
										$LANG_VARS['check_in_reminder_not_for_items'],
	 								    $HTTP_VARS['more_information'],
										$success_items_rs, 
										$failure_items_rs,
										TRUE);
							}
						}
						else
						{
							echo _theme_error($LANG_VARS['undefined_error']);
						}
					}
					else
					{
						echo _theme_error($LANG_VARS['operation_not_available']);
					}
				}
				else if($HTTP_VARS['op'] == 'extension')
				{
					echo _theme_header($LANG_VARS['borrow_duration_extension']);
					echo("<h2>".$LANG_VARS['borrow_duration_extension']."</h2>");
	
					if($CONFIG_VARS['borrow.duration_support']!==FALSE)
					{				
						// It is easier to assume an array in all cases.	
						if(is_checked_listing_vars($HTTP_VARS, 'sequence_number'))
						{
							$listingObject = new HTML_Listing($PHP_SELF, $HTTP_VARS);
							$listingObject->setCheckBoxColumnAndInitList('sequence_number');
						
							$HTTP_VARS['sequence_number'] = $listingObject->getCheckedList();
							$sequence_number_r = $HTTP_VARS['sequence_number'];
										
							// Don't need the HTML_Listings object anymore.
							unset($listingObject);
						}
						else if(is_numeric($HTTP_VARS['sequence_number']))//is_numeric
						{
							$sequence_number_r[] = $HTTP_VARS['sequence_number']; //convert to array here.
						}
						
						if(is_array($sequence_number_r))
						{	
							// If we are providing for custom email to go along with the message then we need to take care of it here.
							if(!is_numeric($HTTP_VARS['default_borrow_duration']) || $HTTP_VARS['more_info_requested'] != 'true')
							{
								// If more info has already been requested, then the 
								// default_borrow_duration must have not been specified
								if($HTTP_VARS['more_info_requested'] == 'true')
								{
									echo format_error_block($LANG_VARS['borrow_duration_extension_must_be_specified']);
								}
								more_information_form('extension', 
											$sequence_number_r, 
											$HTTP_VARS,
											TRUE);
							}
							else
							{
								while ( list(,$sequence_number) = @each($sequence_number_r))
								{
									if(handle_extension($sequence_number, $HTTP_VARS['default_borrow_duration'], $errors))
										$success_items_rs[] = fetch_borrowed_item_r($sequence_number,TRUE);
									else
										$failure_items_rs[] = add_errors_to_borrowed_item_r(fetch_borrowed_item_pk_r($sequence_number), $errors);
								}
	
								display_borrow_results(
										$HTTP_VARS['op'],
										$HTTP_VARS['mode'],
										$LANG_VARS['borrow_duration_extension'],
										$LANG_VARS['borrow_duration_extension_for_items'],
										$LANG_VARS['borrow_duration_extension_not_for_items'],
									    $HTTP_VARS['more_information'],
										$success_items_rs, 
										$failure_items_rs,
										TRUE);
							}
						}
						else
						{
							echo _theme_error($LANG_VARS['undefined_error']);
						}
					}
					else
					{
						echo _theme_error($LANG_VARS['operation_not_available']);
					}
				}
				else
				{
					echo _theme_header($LANG_VARS['operation_not_available']);
					echo _theme_error($LANG_VARS['operation_not_available']);
				}
				
				// Include a link no matter what, because they might have initiated the action by accident.
				if($HTTP_VARS['item_link'] === 'y' && is_numeric($HTTP_VARS['item_id']) && is_numeric($HTTP_VARS['instance_no']))
				{
					$footer_links_r[] = array(url=>"item_display.php?item_id=".$HTTP_VARS['item_id']."&instance_no=".$HTTP_VARS['instance_no']."&listing_link=".$HTTP_VARS['listing_link'],text=>$LANG_VARS['back_to_item']);
				}
				if($HTTP_VARS['listing_link'] === 'y' && is_not_empty_array($HTTP_SESSION_VARS['listing_url_vars']))
				{
					$footer_links_r[] = array(url=>"listings.php?".get_url_string($HTTP_SESSION_VARS['listing_url_vars']),text=>$LANG_VARS['back_to_listing']);
				}
			
				echo format_footer_links($footer_links_r);
				echo _theme_footer();
			}//no guests allowed!
			else
			{
				echo _theme_header($LANG_VARS['not_authorized_to_page']);
				echo _theme_error($LANG_VARS['not_authorized_to_page']);
				echo _theme_footer();
			}
		}//borrow functionality disabled.
		else
		{
			echo _theme_header($LANG_VARS['borrow_not_supported']);
			echo _theme_error($LANG_VARS['borrow_not_supported']);
			echo _theme_footer();
		}
	}
	else
	{
		// invalid login, so login instead.
		include("./login.php");
	}
}//if($CONFIG_VARS['site.enable']!==FALSE)
else
{
	echo _theme_header($LANG_VARS['site_is_disabled'], FALSE);
	echo _theme_error($LANG_VARS['site_is_disabled']);
	echo _theme_footer();
}

// Cleanup after begin.inc.php
require_once("./include/end.inc.php");
?>
